SystemOrganization addCategory: #'Tric-Unloadable'!

Object subclass: #UnloadableClass
	instanceVariableNames: 'name messages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tric-Unloadable'!

!UnloadableClass commentStamp: '<historical>' prior: 0!
Keeps the unloadable bits together for classes that aren't in the image (yet)!

!UnloadableClass methodsFor: 'as yet unclassified' stamp: 'CdG 2/5/2005 22:54'!
addMessage: aString
	(messages includes: aString) ifFalse: [messages add: aString]! !

!UnloadableClass methodsFor: 'as yet unclassified' stamp: 'CdG 2/5/2005 22:30'!
initialize
	messages _ OrderedCollection new! !

!UnloadableClass methodsFor: 'as yet unclassified' stamp: 'CdG 2/5/2005 23:10'!
loadDefinitions
	"Did I mention the Quick&Dirty bit?"
	| scanner |
	scanner _ nil.
	messages do: [:each |
		(each asString findString: 'methodsFor') > 0 
			ifTrue: [
				scanner _ Compiler evaluate: each logged: false]
			ifFalse: [
				scanner theClass
					compile: each
					classified: scanner category
					withStamp: scanner changeStamp
					notifying: nil]]! !

!UnloadableClass methodsFor: 'as yet unclassified' stamp: 'CdG 2/5/2005 22:30'!
name: aString
	name _ aString! !

!UndefinedObject methodsFor: '*tric-unloadable' stamp: 'CdG 2/5/2005 22:53'!
methodsFor: aString stamp: aString2 
	| sourceText |
	sourceText _ thisContext sender methodNode sourceText.
	sourceText _ sourceText copyFrom: 9 to: sourceText size.
	(UndeclaredReceiverOnFileIn new sourceText: sourceText) signal.
	^self! !

!UndefinedObject methodsFor: '*tric-unloadable' stamp: 'CdG 2/5/2005 22:53'!
scanFrom: aStandardFileStream 
	| chunk |
	[(chunk _ aStandardFileStream nextChunkText) size > 0]
		whileTrue:
		[(UnloadableMethodOnFileIn new sourceText: chunk) signal.]! !

!SystemDictionary methodsFor: '*tric-unloadable' stamp: 'CdG 2/5/2005 23:02'!
cleanupUnloadable
	Unloadable keys do: [:each |
		(self includesKey: each) ifTrue: [
			(Unloadable at: each) loadDefinitions.
			Unloadable removeKey: each]].! !

Notification subclass: #UnloadableDefinitionOnFileIn
	instanceVariableNames: 'sourceText'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tric-Unloadable'!

!UnloadableDefinitionOnFileIn commentStamp: '<historical>' prior: 0!
Common superclass for notifications thrown upon encountering stuff for classes that aren't in the image (yet) on fileIn!

UnloadableDefinitionOnFileIn subclass: #UndeclaredReceiverOnFileIn
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tric-Unloadable'!

!UndeclaredReceiverOnFileIn commentStamp: '<historical>' prior: 0!
Notification signaled when a #methodsFor:... is seen in a fileIn for a class that's not present!

!UndeclaredReceiverOnFileIn methodsFor: 'as yet unclassified' stamp: 'CdG 2/5/2005 22:44'!
addToUnloadableDefinitionsWithCurrentTarget: anUnloadableClass
	| target |
	Unloadable ifNil: [Unloadable _ IdentityDictionary new].
	target _ Unloadable at: self targetName 
		ifAbsentPut: [UnloadableClass new name: self targetName].
	"add the source string as a method for now - we can deal with classification later"
	target addMessage: self sourceText.
	^target! !

!UndeclaredReceiverOnFileIn methodsFor: 'as yet unclassified' stamp: 'CdG 2/5/2005 22:44'!
targetName 
	^(sourceText findTokens: ' ') first asSymbol! !

!UnloadableDefinitionOnFileIn methodsFor: 'as yet unclassified' stamp: 'CdG 2/5/2005 22:34'!
addToUnloadableDefinitionsWithCurrentTarget: anUnloadableClass
	self subclassResponsibility! !

!UnloadableDefinitionOnFileIn methodsFor: 'as yet unclassified' stamp: 'CdG 2/5/2005 22:20'!
sourceText
	^sourceText! !

!UnloadableDefinitionOnFileIn methodsFor: 'as yet unclassified' stamp: 'CdG 2/5/2005 22:20'!
sourceText: aString
	sourceText _ aString! !

UnloadableDefinitionOnFileIn subclass: #UnloadableMethodOnFileIn
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tric-Unloadable'!

!UnloadableMethodOnFileIn commentStamp: '<historical>' prior: 0!
Notification signaled when a method definition is seen in a fileIn for a class that's not present!

!UnloadableMethodOnFileIn methodsFor: 'as yet unclassified' stamp: 'CdG 2/5/2005 22:45'!
addToUnloadableDefinitionsWithCurrentTarget: anUnloadableClass
	anUnloadableClass addMessage: sourceText.
	^anUnloadableClass! !

!PositionableStream methodsFor: '*tric-unloadable' stamp: 'CdG 2/5/2005 22:24'!
fileInAnnouncing: announcement 
	"This is special for reading expressions from text that has been formatted 
	with exclamation delimitors. The expressions are read and passed to the 
	Compiler. Answer the result of compilation.  Put up a progress report with
     the given announcement as the title."

	| val chunk unloadableDefinitions |
	unloadableDefinitions _ OrderedCollection new.
	announcement 
		displayProgressAt: Sensor cursorPoint
		from: 0
		to: self size
		during: 
			[:bar | 
			[self atEnd] whileFalse: 
					[[bar value: self position.
					self skipSeparators.
					
					[val := (self peekFor: $!!) 
								ifTrue: [(Compiler evaluate: self nextChunk logged: false) scanFrom: self]
								ifFalse: 
									[chunk := self nextChunk.
									self checkForPreamble: chunk.
									Compiler evaluate: chunk logged: true]] 
							on: InMidstOfFileinNotification
							do: [:ex | ex resume: true].
					self skipStyleChunk]
						on: UnloadableDefinitionOnFileIn
						do: [:ex | unloadableDefinitions add: ex. ex resume]].
			self close].
	self processUnloadableDefinitions: unloadableDefinitions.
	"Note:  The main purpose of this banner is to flush the changes file."
	SmalltalkImage current logChange: '----End fileIn of ' , self name , '----'.
	self flag: #ThisMethodShouldNotBeThere.	"sd"
	Smalltalk forgetDoIts.
	^val! !

!PositionableStream methodsFor: '*tric-unloadable' stamp: 'CdG 2/5/2005 23:02'!
processUnloadableDefinitions: aCollection
	| targetClass |
	"Step one: add all notifications to Unloadable"
	targetClass _ nil.
	aCollection do: [:each |
		targetClass _ each addToUnloadableDefinitionsWithCurrentTarget: targetClass].
	Unloadable isEmpty ifTrue: [^self].
	"Step two: see whether we can load stuff from Unloadable"
	Smalltalk cleanupUnloadable.
	"Step three: inspect Unloadable if there's stuff in it"
	Unloadable isEmpty ifFalse: [Unloadable inspect].
	! !
